@@ -1,10 +1,10 @@
-
-
-from pytorch_gleam.data.collators.multi_sequence import MultiSequenceBatchCollator
-from pytorch_gleam.data.collators.kbi import KbiBatchCollator
-from pytorch_gleam.data.collators.nli_text import NliTextBatchCollator
 from pytorch_gleam.data.collators.base_collators import BatchCollator
-from pytorch_gleam.data.collators.multi_label import MultiLabelBatchCollator
+from pytorch_gleam.data.collators.bert_pre import BertPreBatchCollator
+from pytorch_gleam.data.collators.kbi import KbiBatchCollator
 from pytorch_gleam.data.collators.multi_class_frame import MultiClassFrameBatchCollator
 from pytorch_gleam.data.collators.multi_class_frame_edge import MultiClassFrameEdgeBatchCollator
 from pytorch_gleam.data.collators.multi_class_frame_edge_morality import MultiClassFrameEdgeMoralityBatchCollator
+from pytorch_gleam.data.collators.multi_label import MultiLabelBatchCollator
+from pytorch_gleam.data.collators.multi_sequence import MultiSequenceBatchCollator
+from pytorch_gleam.data.collators.nli_text import NliTextBatchCollator
+from pytorch_gleam.data.collators.sequence_to_sequence import SequenceToSequenceBatchCollator
